#!perl

package PureMessage::List::Base::cfg;

use strict;
use warnings;
use Carp qw(croak);

use PureMessage::Resources::DBH qw(DBH);
use PureMessage::Resources::Manager;

return 1;

sub new {
    my($class, $source, %conf) = @_;
	
    $conf{dbh} ||= DBH;
    $conf{resource_name} = $source;
    # strip cfg: / file: / etc from front of resource name
    $conf{resource_name} =~ s/^.+?://;
    $conf{source} ||= $source;
    $conf{items} ||= [];
    # $conf{sth} = $conf{dbh}->prepare($SQL_MATCH_LIST);
    
    my $self = bless \%conf, $class;
    $self->refresh;
    $self;
}

sub _change_level {
    my $self = shift;
    my($change_level) = ($self->data->{dbh}->selectrow_array(
        "SELECT change_id FROM cfg_instance WHERE instance_id = ?",
        {}, $self->data->{instance_id}
    )) or
        croak "Failed to query database for change level of instance #",
	    $self->data->{instance_id}, ": ", $self->data->{dbh}->errstr;

    $change_level;
}

sub refresh {
    my $self = shift;
    my $change_level = $self->_change_level;
    return
        if ($self->{change_level} && $self->{change_level} == $change_level);
    $self->{change_level} = $change_level;
	
    my $items = $self->_parse_db(
        $self->data->{instance_id}, $self->data->{userkey}, $self->{dbh}
    );
    $self->items($items);
    $self;
}

# initialize the database for this list. We may need to make a resource and
# we'll need an instance
sub _create {
   my ($CLASS,$source,%options) = @_;
   croak("The source format '$source' is unknown") unless $source =~ /^(cfg):(.+)$/;
   my $init_values = $2;
   # create a resource if it doesn't exist
   my $rm = PureMessage::Resources::Manager->new(); 
   my $found_res = $rm->Find($options{id});
   
   unless($found_res) {
      # retrieve attributes and package them up for the Define call 
      my %attributes;
      $attributes{$_} = $options{$_} foreach qw( description name match_type comparator );
      die "Unable to create resource" unless $rm->Define(name => $options{id},type=>$CLASS->_type,attributes => \%attributes);
   }
   $rm->add($options{id});
   return (%options);
}
