#!perl

package PureMessage::ListData::Base::cfg;

use strict;
use warnings;

use PureMessage::ListData::Base;
use PureMessage::Resources::DBH qw(DBH);
use Carp qw(croak);

use base qw(PureMessage::ListData::Base);

return 1;

sub new {
    my $class = shift;
    my $source = shift;
    my $self = bless { @_ }, $class;

    croak ref($self), " object created without an instance_id"
        unless($self->{instance_id});
    
    $self->{dbh} ||= DBH;

    if(!$self->{resource_name}) {
        $self->{resource_name} = $source;
        $self->{resource_name} =~ s/^(.+?)://;
    }
    
    # regular lists look themselves up by instance_id;
    # children of MultiLists already have an instance_id, and look
    # themselves up by userkey.
    
    if(!$self->{_ml_instance_id} && !$self->{_ll_instance_id}) {
        $self->{_ll_instance_id} = $self->{instance_id};
    }

    if($self->{_ml_instance_id}) {
        $self->{userkey} = $self->{resource_name};
    } else {
        $self->{userkey} = '';
    }

    $self;    
}

## internal methods

sub do_or_die {
    my($self, $query, @args) = @_;
    my $sql_error = 0;
    eval { $self->{dbh}->do($query, {}, @args) or $sql_error++ };
    $sql_error++ if $@;
    if ($sql_error) {
        (my $q = $query) =~ s/\s{2,}/ /sg;
        croak($q,
              (@args) ? "["  . join(',', map { "'$_'" } @args) . "]" : '', 
              "\n" . $self->{dbh}->errstr);
    }
}

sub commit {
    my $self = shift;
    return unless $self->_modified;
        
    my $affected = 0;
    
    my $to_add = delete $self->{to_add};
    my $to_add_order = delete $self->{to_add_order};
    my $to_remove = delete $self->{to_remove};
    my $clear = delete $self->{clear};
    my $delete_source = delete $self->{delete_source};

    my $num_to_add = scalar keys %$to_add;

    $self->{dbh}->begin_work if($self->{dbh}->{AutoCommit});
    
    if($clear || $delete_source) {
        my $rows = $self->commit_clear;
        $affected += $rows if(substr($rows, 0, 1) ne '0');

        if($delete_source) {
            $self->commit_delete_source;
            $affected++;
        }
    }

    unless($delete_source) {
        if($to_remove && %$to_remove) {
            foreach my $entry (values(%$to_remove)) {
                $self->commit_remove($entry);
                $affected++;
            }
        }
        
        ### NOTE[tylerm]: this is the intention of the ::file handler's
        ### $to_add_order... but the underlying psql tables don't have
        ### any support for ordering...!
        
        if($to_add && %$to_add) {
            my(@order);
            if($to_add_order && @$to_add_order) {
                @order = @$to_add_order;
            } else {
                @order = sort(keys(%$to_add));
            }
            
            foreach my $entry (grep($to_add->{"$_"}, @order)) {
                $self->commit_add($entry);
                $affected++;
            }
        }
    }
    
    $self->{dbh}->commit() or die ref($self->{dbh}), "->commit() failed: ",
        $self->{dbh}->errstr;

    $affected;
}
