#!perl

use strict;
use warnings;

package PureMessage::ListData::cfg;

use PureMessage::ListData::Base::cfg;
use PureMessage::ListData::file;
use PureMessage::Resources::DBH qw(DBH);
use base qw(PureMessage::ListData::Base::cfg PureMessage::ListData);

use Carp qw(croak);

return 1;

### enumerate

sub enumerate_items {
    my($self, $cb, @args) = @_;
    if(my $sth = $self->{dbh}->prepare_cached(
        q{
            SELECT list_value FROM cfg_listrow WHERE instance_id = ?
            AND userkey = ?
        }
    )) {
        my $x;
        $sth->execute($self->{instance_id}, $self->{userkey});
        while($x = $sth->fetchrow_arrayref) {
            $cb->($x->[0], @args) if(defined($x->[0]) && length($x->[0]));
        }
        $sth->finish;
    } else {
        die ref($self->{dbh}), "->prepare_cached failed: ",
            $self->{dbh}->errstr;
    }
}

### commit

sub commit_clear {
    my $self = shift;
    $self->do_or_die(
        q{DELETE FROM cfg_listrow WHERE instance_id = ? AND userkey = ?},
        $self->{instance_id}, $self->{userkey}
    );
}

sub commit_delete_source {
    my $self = shift;
    if(!$self->{userkey}) {
        $self->do_or_die(
            q{DELETE FROM cfg_listrow WHERE instance_id = ?},
            $self->{instance_id}
        );
    }
}

sub commit_remove {
    my($self, $key) = @_;
    
    $self->do_or_die(
        q{
            DELETE FROM cfg_listrow WHERE instance_id = ? AND userkey = ?
            AND list_value = ?
        },
        $self->{instance_id}, $self->{userkey}, $key
    );
}

sub commit_add {
    my($self, $key) = @_;
    
    $self->do_or_die(
        q{
            INSERT INTO cfg_listrow (instance_id, userkey, list_value)
            VALUES (?, ?, ?)
        },
        $self->{instance_id}, $self->{userkey}, $key
    ) if(defined($key) && length($key));
    
    $1;
}
