#!perl

package PureMessage::Map::cfg;

use strict;
use PureMessage::MapData::cfg;
use PureMessage::Map::Items;
use PureMessage::List::Base::cfg;
use base qw(PureMessage::List::Base::cfg PureMessage::Map::Items);
use Carp qw(croak);

sub _type { "map"; }

sub _parse_db {
    my($class, $instance, $userkey, $dbh) = @_;
    if(my $sth = $dbh->prepare_cached(q{
        SELECT k_name, k_value FROM cfg_kv WHERE instance_id = ? AND userkey = ?
    })) {
        $sth->execute($instance, $userkey);
        my(@items);
		my($k, $v);
        while(($k, $v) = ($sth->fetchrow_array)) {
            push(@items, PureMessage::MapData::cfg->_entry($k, $v));
        }
        return \@items;
    } else {
        die ref($dbh), "->prepare_cached failed: ", $dbh->errstr;
    }
}

sub data {
    my $self = shift;
    
    unless($self->{mapdata}) {
        croak "Need source parameter to edit list data" unless($self->{source});
        $self->{mapdata} =
            PureMessage::MapData::cfg->new($self->{source}, %$self, @_);
    }

    $self->{mapdata};
}

# initialize the database for this list. We may need to make a resource and
# we'll need an instance
sub _create {
   my ($CLASS,$source,%options) = @_;
   croak("The source format '$source' is unknown") unless $source =~ /^(cfg):(.+)$/;
   my $init_values = $2;
   # create a resource if it doesn't exist
   my $rm = PureMessage::Resources::Manager->new(); 
   my $found_res = $rm->Find($options{id});
   
   unless($found_res) {
      # retrieve attributes and package them up for the Define call 
      my %attributes;
      $attributes{$_} = $options{$_} foreach qw( description name match_type comparator );
      die "Unable to create resource" unless $rm->Define(name => $options{id},type=>'map',attributes => \%attributes);
   }
   $rm->add($options{id});
   return (%options);
}

1;
