package PureMessage::MapData::cfg;

use strict;
use warnings;
use Carp qw(croak);
use DBI;

use PureMessage::Resources::DBH qw(DBH);
use PureMessage::MapData::Entry;
use PureMessage::MapData::Entry::Item;
use PureMessage::ListData::Base::cfg;
use PureMessage::MapData::file;

use base qw(PureMessage::ListData::Base::cfg);

*_as_item = *PureMessage::MapData::file::_as_item;

# keys are stored unescaped in the db for matching purposes;
# values are stored escaped because k_value is not an array.
# in the future it might be better to have a separate mapping table
# with an array type for value
sub _entry {
	my($self, $left, $right) = @_;
	PureMessage::MapData::Entry->parse(join(
	    ": ", PureMessage::MapData::Entry::Item->new($left)->as_string,
		$right	
	));
}

sub _unentry {
	my($self, $entry) = ($_[0], $_[0]->_as_item($_[1]));
	($entry->left->value, join(" ", map { $_->as_string } $entry->right));
}

### enumerate

sub enumerate_items {
    my($self, $cb, @args) = @_;
    if(my $sth = $self->{dbh}->prepare_cached(
        q{
            SELECT k_name, k_value FROM cfg_kv WHERE instance_id = ?
            AND userkey = ?
        }
    )) {
        my($k, $v);
        $sth->execute($self->{instance_id}, $self->{userkey});
        while(($k, $v) = ($sth->fetchrow_array)) {
            $cb->($self->_entry($k, $v), @args);
        }
        $sth->finish;
    } else {
        die ref($self->{dbh}), "->prepare_cached failed: ",
            $self->{dbh}->errstr;
    }
}

### commit

sub commit_clear {
    my $self = shift;
    $self->do_or_die(
        q{DELETE FROM cfg_kv WHERE instance_id = ? AND userkey = ?},
        $self->{instance_id}, $self->{userkey}
    );
}

sub commit_delete_source {
    my $self = shift;
    if(!$self->{userkey}) {
        $self->do_or_die(
            q{DELETE FROM cfg_kv WHERE instance_id = ?},
            $self->{instance_id}
        );
    }
}

sub commit_remove {
    my($self, $entry) = @_;
    
    $self->do_or_die(
        q{
            DELETE FROM cfg_kv WHERE instance_id = ? AND userkey = ?
            AND k_name = ?
        },
        $self->{instance_id}, $self->{userkey}, $entry->left->value
    );
}

sub commit_add {
    my($self, $entry) = @_;
    
    $self->do_or_die(
        q{
            INSERT INTO cfg_kv (instance_id, userkey, k_name, k_value)
            VALUES (?, ?, ?, ?)
        },
        $self->{instance_id}, $self->{userkey}, $self->_unentry($entry)
    );
    
    $1;
}

1;
