package PureMessage::MultiList::cfg;

use strict;
use warnings;

use Carp;
use DBI;
use PureMessage::List::cfg;
use PureMessage::ListData::cfg;
use PureMessage::Resources::DBH qw(DBH);
use PureMessage::MultiList;
use base qw(PureMessage::MultiList);

# new(source, confopt...)
#
# find_list({key => (user name), edit_mode => (true:create)})
#
# enumerate_keys($cb, {cb_arg=>, }
#	... apply a codeblock with arg to each key
#
# refresh() foreach enduser in _mf_list, 

sub new {
    my $class = shift;
    my $source = shift;
    my $self = bless { @_ }, $class;

    croak ref($self), " object created without an instance_id"
        unless($self->{instance_id});

    $self->{dbh} ||= DBH;

    $self->{profile_id} = 0 unless(defined($self->{profile_id}));
    if(!$self->{resource_name}) {
        $self->{resource_name} = $source;
        $self->{resource_name} =~ s/^(.+?)://;
    }
    
    if(!$self->{_ml_instance_id}) {
        $self->{_ml_instance_id} = $self->{instance_id};
    }

    $self->{_mf_lists} ||= {};
    $self;    
}

sub find_list {
    # "creation" of a multilist's child list is implicit in the database
    # with the addition of a key with that "userkey" value.
    my $self = shift;
    my %opts = @_;
    my $key = $opts{key};
    my %child_args = (%$self, source => $key);
    delete($child_args{dbh});
    croak "parameter 'key' required" unless defined $key;
    my $lists = $self->{_mf_lists};
    $lists->{$key} ||= PureMessage::List::cfg->new($key, %child_args);
    return $lists->{$key} || undef;
}

sub enumerate_keys {
    my $self = shift;
    my $cb = shift;
    my %opts = @_;
    my $cb_arg = $opts{cb_arg};
    
    if(my $sth = $self->{dbh}->prepare_cached(q{
        SELECT DISTINCT userkey FROM cfg_listrow
        WHERE instance_id = ? 
    })) {
        $sth->execute($self->{instance_id});
        while(my($key) = ($sth->fetchrow_array)) {
            $cb->($cb_arg ? ($cb_arg, $key) : $key);
        }
        $sth->finish();
    } else {
        die ref($self->{dbh}), "->prepare_cached failed: ",
            $self->{dbh}->errstr;
    }
}

sub refresh {
    my $self = shift;
    my $lists = $self->{_mf_lists};
    foreach (keys %$lists) {
        $lists->{$_}->refresh;
    }
}

sub _abs_file {
    my $class = shift;
    my $file = shift;
    return list_absfile($file);
}

1;
