package PureMessage::Resources::ASData::Base;
use ActiveState::Config qw(:path);
use base qw(Exporter);
our @EXPORT = qw(_as_paths_undo as_paths _descend);
use strict;
use warnings;

sub new {
  die "This needs to be overriden";
}

sub reset {
}

sub _validate {
   my $self = shift;
   return if $self->{attributes}->{ignore_errors};
   if(exists($self->{attributes}->{validate}) and $self->{attributes}->{validate}) {
       my @validators = ref($self->{attributes}->{validate}) eq 'ARRAY' 
                      ? @{$self->{attributes}->{validate}} 
                      : ($self->{attributes}->{validate});
       foreach my $validator (@validators) {
           eval {
               eval("require $validator");
               die "Error creating validator '$validator': $@\n" if $@;
               if ($validator->can('validate')) {
                   $validator->validate($self->{_as_config});
               } elsif($validator->can('define_core_validators')) {
                   $validator->validate($self->{_as_config});
               } else {
                   die "$validator has no validate methods\n";
               }
           };
           if($@) {
              warn "Error validating config file '$self->{_as_data_name}' with '$validator' validator: $@\n";
           }
           
       }
   }
}

sub _init_before_use {
   my $self = shift;
}

sub as_hash {
   my $self = shift;
   $self->_init_before_use();
   $self->{_as_config}->as_hash();
}

sub as_hash_paths {
  my ($self) = @_;
  return $self->as_paths($self->as());
}


sub root {
    my $self = shift;
    $self->_root_node($self->{_as_config});
}

sub typedecl {
   my ($self,@options) = @_;
   $self->{_as_config}->typedecl(@options);
}

sub default {
   my ($self,@options) = @_;
   $self->{_as_config}->default(@options);
}


sub parse_file {
}

sub save {
}

sub as {
   my $self = shift;
   $self->_init_before_use();
   return $self->{_as_config};
}


sub import_as {
  my ($self,$as) = @_;
  $self->_init_before_use();
  my $paths = $self->as_paths($as);
  my $root = $self->_real_root();
  foreach my $path (sort keys %$paths) {
      #path_node($root,$path,delete => 1);
      path_value($root,$path,$paths->{$path});
  }
}


sub import_file {
    my ($self,$filename) = @_;
    my $as = ActiveState::Config->new( allow_includes => 1);
    $as->parse_file($filename);
    $self->import_as($as);
}


sub _as_paths_undo {
    my $r = shift;
    my %result;
    foreach my $path ( keys %$r) {
        my @parts = split(/\./,$path);
        my $cur = \%result;
        my $key = '';
        foreach my $part (@parts) {
           if($part =~ /^([^#]+)#([^#]+)$/) {
              $cur->{$key}->{$1}->{$2} = $cur->{$key}->{$1}->{$2} || {};
              $cur = \%{$cur->{$key}->{$1}};
              $key = $2;
           } else {
              $cur->{$key}->{$part} = $cur->{$key}->{$part} || {};
              $cur = \%{$cur->{$key}};
              $key = $part;
           }
        }
        $cur->{$key} = $r->{$path};
    }
    return $result{''};
}

sub as_paths {
  my($self, $as) = @_;
  my $tree = $self->_root_node($as);
  my $result = {};
  _descend($tree,$result);
  return $result;
}


sub changed_as_paths {
  my($self, $as) = @_;
  my $tree = $self->_root_node($as);
  my $result = {};
  _descend_changed($tree,$result);
  return $result;
}

sub mark_fresh {
  my($self, $as) = @_;
  my $tree = $self->_root_node($as);
  _descend_fresh($tree);
  return 1;    
}

sub _do_descend {
    my ($node, $cb) = @_;
    if ($node->children()) {
        my @n = $node->children();
        foreach my $n (@n) {
            _do_descend($n, $cb);
        }
    } else {
        $cb->($node);
    }
}

sub _descend {
    my($node, $ref) = @_;
    _do_descend(
        $node,
        sub {
            my $n = shift;
            $ref->{$n->{path}} = $n->value if $n->{path} !~ /!uniqid/;
        }
    );
}

sub _descend_fresh {
    my $node = shift;

    _do_descend(
        $node,
        sub { my $n = shift; $n->changed(0); }
    );
}

sub _descend_changed {
    my($node, $ref) = @_;
    _do_descend(
        $node,
        sub {
            my $n = shift;
            
            if(($n->{path} !~ /!uniqid/) && ($n->changed)) {
                $ref->{$n->{path}} = $n->value;
            }
        }
    );
}

sub _root_node {
   my($self, $as) = @_;
   if($self->{attributes}->{slice}) {
       my(%opts) = ();
       if($self->{attributes}->{slice_create}) {
           $opts{create} = $self->{attributes}->{slice_create};
       }
       path_node(_real_root($as), $self->{attributes}->{slice}, %opts);
   } else {
       _real_root($as);
  }
}

sub _real_root {
    my $as = shift;
    if($as->isa("PureMessage::Resources::ASData::Base")) {
        $as->as->root;
    } else {
        $as->root;
    }
}


1;
