package PureMessage::Resources::ASData::DB;
use ActiveState::Config qw(:path);
use PureMessage::Resources::KVData::DB;
use PureMessage::Resources::ASData::Base;
use base qw(PureMessage::Resources::ASData::Base);

sub new {
   my ($CLASS,$name,$instance_id,%attributes) = @_;
   my $self = { _as_data_name => $name, _as_data_iid => $instance_id, attributes => \%attributes };
   $self->{_as_data_kv} = PureMessage::Resources::KVData::DB->new($name,$instance_id);
   $self = bless($self,$CLASS);
   $self->refresh();
   $self;
}

sub refresh {
   my $self = shift;
   $self->{_as_config} = ActiveState::Config->new(writable => 1, allow_includes => 1,strict=>$self->{attributes}->{ignore_errors} ? 0 : 1);
   $self->_validate();
   $self->parse_file();
}

sub reset {
   my $self = shift;
   $self->{_as_data_kv}->reset();
   $self->refresh(); 
}

sub _init_before_use {
}

sub parse_file {
    my ($self,@config) = @_;
    my $kv = $self->{_as_data_kv};
    my $d = $kv->hashref();
    my $root = $self->{_as_config}->root();
    my %arrays;
    if ($d and ref($d) eq 'HASH') {
      foreach my $key (keys %$d) {
          if ($d->{$key} eq '__DEFAULT__') {
            path_node($root,$key,delete=>1); # remove that node if it's there so default can fall through
            path_value($root,$key,undef);
          } elsif ($key =~ /([^\^]+)\^(\d+)$/) {
                $arrays{$1} = [] unless exists($arrays{$1});
                push(@{$arrays{$1}},$d->{$key});
          } else {
            path_value($root,$key,$d->{$key});
          }
      }
      foreach my $a (keys %arrays) {
            path_value($root,$a,$arrays{$a});
      }
    }
    $self->{arrays} = \%arrays;
    $self->mark_fresh($self->{_as_config});
    return 1;
}

sub save {
  my $self = shift;
  $self->_import_as_to_db($self->{_as_config});
}

sub import_file {
    my ($self,$filename) = @_;
    my $as = ActiveState::Config->new( allow_includes => 1);
    $as->parse_file($filename);
    $self->import_as($as);
}

sub _import_as_to_db {
  my ($self,$as) = @_;
  my $paths = $self->changed_as_paths($as);
  my $kv = $self->{_as_data_kv};
  foreach my $path (keys %$paths) {
      $paths->{$path} = '__DEFAULT__' if $paths->{$path} eq undef;
      if(ref($paths->{$path}) eq 'ARRAY') {
          # clean out the previous values if they are there
          if (exists($self->{arrays}->{$path})) {
            my $av = $self->{arrays}->{$path};
            for (my $i = 0; $i < scalar(@$av); $i++) {
              $kv->delete("$path^$i");
            }
          }
          my $i=0;
          foreach my $v (@{$paths->{$path}}) {
              $kv->value("$path^$i",$v);
              $i++;
          }
      } else {
         $kv->value($path,$paths->{$path});
      }
  }
}

1;
