package PureMessage::Resources::ASData::Local;
use ActiveState::Config qw(:path);
use PureMessage::Resources::KVData::DB;
use PureMessage::Version qw( $PREFIX );
use PureMessage::Resources::ASData::Base;
use base qw(PureMessage::Resources::ASData::Base);
use strict;

sub new {
   my ($CLASS,$name,%attributes) = @_;
   # make the filename absolute if it is relative
   my $filename = $attributes{filename} =~ /^[^\/]/ ? "$PREFIX/$attributes{filename}" : $attributes{filename};
   my $self = { _as_data_name => $name, filename => $filename,attributes => \%attributes };
   $self = bless($self,$CLASS);
   $self->refresh();
  $self;
}

sub refresh {
   my $self = shift;
   $self->{_as_config} = ActiveState::Config->new(allow_includes => 1, strict => $self->{attributes}->{ignore_errors} ? 0 : 1);
   $self->_validate();
   $self->{have_initialized} = 0;
   $self->parse_file();
}

sub reset {
   my $self = shift;
   $self->_init_before_use();
   my $paths = $self->as_paths($self->{_as_config});  
   my $root = $self->{_as_config}->root();
   foreach my $path (keys %$paths) {
      path_node($root,$path,delete=>1);
   }
}

sub _init_before_use {
   my $self = shift;
   return if $self->{have_initialized};
   $self->parse_file();
   $self->{have_initialized} = 1;
}


sub parse_file {
    my ($self,$filename) = @_;
    return if $self->{have_initialized};
    $self->{have_initialized} = 1;
    my %options = ();
    $options{writable} = 1 if defined($self->{attributes}->{writable}) and $self->{attributes}->{writable};
    return $self->{_as_config}->parse_file($self->{filename},%options);
}

sub save {
  my $self = shift;
  $self->{_as_config}->save(); # doing this will invalidate our asconfig object
  delete($self->{_as_config});
  $self->refresh();
}

sub import_file {
    my ($self,$filename) = @_;
    my $as = ActiveState::Config->new( allow_includes => 1);
    $as->parse_file($filename);
    $self->import_as($as);
}

1;
