package PureMessage::Resources::Config;
use strict;
use ActiveState::Config qw(:path);
use PureMessage::Version qw($PREFIX);
use ActiveState::Config::Simple qw(config_parse_file);
use base qw(PureMessage::Config::Validator Exporter);



=head1 NAME

PureMessage::Resources::Config - Resource Configuration Management

=head1 SYNOPSIS

    use PureMessage::Resources::Config qw( %resources_config );

    print $resources_config{default};

=head1 DESCRIPTION


The C<PureMessage::Resources::Config> package alows editing and
discovery of resource related configuration data. Importing the
hash C<%config> or it's alias C<%resources_config> allows accessing
of configured values. The following class method are provided.

=over

=cut


our @EXPORT_OK = qw( %config $object %resources_config);
our %config;

our $resources_config; # alias that doesn't clash with other %configs
*resources_config =\%config;

our $object;
our $real_PREFIX = $ENV{PREFIX} || $PREFIX; # for tests

sub refresh() {
    #require PureMessage::Config::Validator;
    
    #$object = ActiveState::Config->new(strict => 1, allow_includes => 1);
    #PureMessage::Config::Validator->define_core_validators($object);
    #PureMessage::Resources::Config->validate($object);
    #$object->parse_file("$real_PREFIX/etc/resources.conf");
    %config = %{config_parse_file("$real_PREFIX/etc/resources.conf")};
    #%config = $object->as_hash;
    $config{prefix} = $PREFIX;
    \%config; # return this so the refresher can get it
}

refresh();

=item edit()

 Will open this config as a writeable  ActiveState::Config object
 and return it. This can die when configuration file has invalid
 syntax.

=cut

sub edit {
   my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
   PureMessage::Resources::Config->validate($p);
   $p->parse_file("$PREFIX/etc/resources.conf",writable => 1);
   return $p;
}

sub validate {
    my ($pkg, $p) = @_;
    $p->typedecl('default', '#str');
    $p->default('default','local'); #  
    $p->default('max_item_change','10000'); #  
    $p->typedecl('resource#*','#any');
    $p->typedecl('resource#*.*','#any');
    $p->typedecl('resource#*.attributes.*','#any');
    $p->typedecl('import*','#any');
    $p->typedecl('export*','#any');
    $p->typedecl('max_item_change','#int');
    $pkg->include_sub_validators($p);
}

1;



=back

=head1 COPYRIGHT

Copyright (C) 2004, Sophos PLC

=head1 SEE ALSO
 
 none
  
=cut

