package PureMessage::Resources::Resource;
use PureMessage::Resources::Manager;
use File::Spec::Functions qw(catdir);

use strict;

# returns an array of all available resources types.  This is just
# a listing of all lowercase names in the Resources::Resource::Resource/* 
# dir
#

my %IMPLEMENTATIONS;

sub Available {
   %IMPLEMENTATIONS = _get_available() unless keys(%IMPLEMENTATIONS);
   return keys(%IMPLEMENTATIONS) if wantarray;
   return \%IMPLEMENTATIONS;
}


sub _get_available {
    my $type = shift || '';
    my %imps;
    for (@INC) {
        my $dir = catdir($_, split("::", __PACKAGE__),$type);
        next unless opendir(my $d, $dir);
        while (defined(my $f = readdir($d))) {
            next unless $f =~ /^([a-z][a-z_0-9]+)\.pm$/;
            $imps{$1} = 0;
        }
    }
    return %imps;
}


sub new {
   my ($CLASS,$store,%options) = @_;
   
   die "Resource Store is invalid" unless $store;
   die  "Resource name missing or undefined" unless exists($options{name});

   unless (defined($options{type}) ) {
      # then we need to determine what type of resources this is
      # so we can create the correct type of class
      my $res_def = PureMessage::Resources::Manager->Find($options{name});
      die "Unable to retrieve a valid resource record for '$options{name}'." unless $res_def->{name};
      # copy all the data we retrieved to the options.. unless it's already there
      foreach my $key (keys %{$res_def}) {
          $options{$key} = $res_def->{$key} unless exists($options{$key});
      }
   } 
   Available(); # just use this to make sure our global is populated correctly
   die "Invalid resource type '$options{type}'" unless exists( $IMPLEMENTATIONS{ $options{type} } );
   
   my $package_name = "PureMessage::Resources::Resource::$options{type}";

   # we haven't loaded it yet so we should do that now... 
   eval "require $package_name"
      or die "Error creating an implementation for a resource of type '$options{type}': $@";

   my $obj;
   $obj = $package_name->new($store,%options);
   
   return $obj if $obj;
   return;
}


1;

