package PureMessage::Resources::Resource::asdata;
use PureMessage::Resources::Resource::Base;
use ActiveState::Config qw(:path);
use base PureMessage::Resources::Resource::Base;
use PureMessage::Version qw($PREFIX);

sub new {
    my ($CLASS,$store,%conf) = @_;
    my $self = bless(\%conf,$CLASS);
    $self->{store} = $store;
    $self;
}

sub create_local {
    my $self = shift;
    my $filename = $self->{attributes}->{filename} =~ /^[^\/]/ 
                 ? "$PREFIX/$self->{attributes}->{filename}" 
                 : $self->{attributes}->{filename};
    return 1 if -e $filename and -f _;
    open FV,">$filename" || die "Unable to create file '$filename' needed by resource '$self->{name}'\n";
    my $date = localtime(time);
    print FV <<EOF;
# Configuration auto created on $date for resource $self->{name}
EOF
    die "Error closing '$filename' needed by resource '$self->{name}'\n" unless close FV;

}

sub _native_local {
    my $self = shift;
    my %options = @_;

    $options{writable} = 1 if !exists($options{writable}) and $self->{store}->writable();
    $options{ignore_errors} = $self->{ignore_errors} ? 1 : 0;

    require PureMessage::Resources::ASData::Local;
    my $res = PureMessage::Resources::ASData::Local->new($self->{name},%options,%{$self->{attributes}});
}

sub _native_db {
    my $self = shift;
    require PureMessage::Resources::Manager;
    my $temp;
    $temp = PureMessage::Resources::Manager->Find($self->{name}) unless exists($self->{attributes}) and scalar(keys(%{$self->{attributes}}));
    if ($temp) {
       foreach my $key (keys %$temp) {
           $self->{$key} = $temp->{$key};
       }
    }
    require PureMessage::Resources::ASData::DB;
    my $res = PureMessage::Resources::ASData::DB->new($self->{name},$self->{instance_id},%{$self->{attributes}},exists($self->{ignore_errors}) ? ( ignore_errors => $self->{ignore_errors} ? 1 : 0 ) : ());
}

sub checksum {
}

sub autocommit {
   my ($self,$autocommit) = @_;
   $self->{autocommit} = $autocommit;
}

sub save{
    my $self = shift;
    $self->{native}->save() if exists($self->{native}) and $self->{native};
}

sub cclear {
    my $self = shift;
    $self->native()->reset();
}

sub cinsert {
    my ($self,$user,$path,$value) = @_;
    my $root = $self->native()->_real_root();
    path_value($root,$path,$value);
}

sub cdelete {
    my ($self,$user,$path,$value) = @_;
    my $root = $self->native()->_real_root();
    path_node($root,$path,delete=> 1);
}

sub cupdate {
    my ($self,$user,$path,$value) = @_;
    my $root = $self->native()->_real_root();
    path_value($root,$path,$value);
}

sub enumerate{
    my ($self,$sub,$change_id) = @_;
    # TODO we have to ignore the change_id for now 
    
    # nieve as heck implementation
    eval {
        my $sync_data = $self->native();
        &$sub($self->cfmt_clear()); # empty the destination
        my $paths = $sync_data->as_hash_paths();

        foreach my $p (keys %$paths) {
            #print "sync: $p = $paths->{$p}\n";
            &$sub($self->cfmt_insert('',$p,$paths->{$p}));
        }
    };
    $self->error($@) if $@;
}

sub _aggregate_op {
  my ($self,$op,$ignore,$path,$value) = @_;
  my $temp_output;
  if ($op eq 'I') {
    if(ref($value) eq 'ARRAY') {
        my $i = 0;
        foreach my $v (@$value) {
          if ( !$i ) {
            $temp_output = $path;
            $i++;
          } else {
            $temp_output =  ' ' x length($path);
          }
          $temp_output .= " = $v";
        }
    } else {
      $temp_output = "$path = $value";
    }
 }
 return ($temp_output);  
}


1;
