package PureMessage::Resources::Resource::kvdata;
use base PureMessage::Resources::Resource::Base;
use strict;
my $KVDATA_DEBUG = 0;

sub new {
    my ($CLASS,$store,%conf) = @_;
    my $self = bless(\%conf,$CLASS);
    $self->{store} = $store;
    $self->{autocommit} = 1;
    $self;
}

sub create_local {
    my $self = shift;
    return 1 if $self->_native_local();
    return;
}

sub _native_local {
    my $self = shift;
    require PureMessage::Resources::KVData::Local;
    my $kvd = PureMessage::Resources::KVData::Local->new($self->{name},autocommit => $self->{autocommit},( $self->{attributes} ? (%{$self->{attributes}}) : ()));
    return $kvd;
}

sub _native_db {
    my $self = shift;
    die "No instance_id defined for this resource, cannot created a native instance" unless $self->{instance_id};
    require PureMessage::Resources::KVData::DB;
    my $kvd = PureMessage::Resources::KVData::DB->new($self->{name},$self->{instance_id});
    return $kvd;
}

sub autocommit {
   my ($self,$auto) = @_;
   $self->{autocommit} = $auto;
   $self->{native}->autocommit($auto) if exists($self->{native}) and $self->{native};
}

sub save {
   my $self  = shift;
   if(exists($self->{native}) and $self->{native}) {
       $self->{native}->save();
       $self->{native}->refresh();
   }
}


use Data::Dumper;

sub enumerate {
    my ($self,$sub,$change_id) = @_;
    # TODO we have to ignore the change_id for now 
    
    # nieve as heck implementation
    eval {
        my $sync_data = $self->native();
        &$sub($self->cfmt_clear()); # empty the destination
        foreach my $user (@{$sync_data->users()}) {
            my $data;
            print "SYNCING USER: '$user'\n" if $KVDATA_DEBUG;
            $data = $sync_data->user_keys($user);
            print "DATA OF: " . Dumper($data) . "\n" if $KVDATA_DEBUG;
            foreach my $key (keys %$data) {
                &$sub($self->cfmt_insert($user,$key,$data->{$key}));
            }
        }
    };
    $self->error($@) if $@;
}

sub cclear {
   my $self = shift;
   my $n = $self->native();
   $n->reset();
}

sub cdelete {
   my $self = shift;
   my ($user,$key) = @_;
   my $n = $self->native();
   if (defined($key)) {
       # delete just this key from the users records
      if($user ne ''and $user) {
          $n->udelete($user,$key);
      } else {
          $n->delete($key);
      }
   } else {
      # delete this user
      $n->delete_user($user);
   }
}

sub cupdate {
   my $self = shift;
   my ($user,$key,$value) = @_;
   my $n = $self->native();
   if($user ne '') {
       $n->uvalue($user,$key,$value);
   } else {
       $n->value($key,$value);
   }
}

sub cinsert {
   my $self = shift;
   my @args = @_;
   $self->cupdate(@args);
}

sub set_changeid {
   my $self = shift;
}


sub _aggregate_op {
  my ($self,$op,$user,$key,$value) = @_;
  if ($op eq 'I') {
      return ("$key = $value");
  }
}



1;




