package PureMessage::Resources::Resource::list;
use PureMessage::Resources::Resource::Base;
use base PureMessage::Resources::Resource::Base;
use PureMessage::Version qw($PREFIX);
use File::Copy;
use Text::Diff;
use strict;
use warnings;


sub can_incremental {
  return 1;
}

sub create_local {
    my $self = shift;
    require PureMessage::List;
    return 1 if PureMessage::List->Available($self->{name});
    # have to add it
    my %extra = ();
    $extra{source} = $self->{attributes}->{source} if exists($self->{attributes}->{source});
    $extra{args}->{resource_controlled} = 'true';
    my %list_config = ( id => $self->{name}, 
                        name => $self->{attributes}->{name} ||
                            $self->{attributes}->{description},
                        description => $self->{attributes}->{description},
                        precious => $self->{attributes}->{precious},
                        match_type => $self->{attributes}->{match_type},
                        %extra,
                      );
    PureMessage::List->Setup(%list_config);
    return 1; # Setup will croak if there is a problem                  
}

sub _native_local {
    my $self = shift;
    # create_local will only create the list if it doesn't already exist
    require PureMessage::List;
    $self->create_local();
    my $list = PureMessage::List->new($self->{name});
    return $list;
}

sub _native_db {
    my $self = shift;
    unless($self->{attributes}) {
      require PureMessage::Resources::Manager;
      my $temp = PureMessage::Resources::Manager->Find($self->{name});
      $self->{attributes} = $temp->{attributes};
    } 
    # don't allow the source param from the resource def to override the 
    # cfg type
    my %custom_attributes = %{$self->{attributes}};
    delete($custom_attributes{source});
    
    require PureMessage::List;
    my $list = PureMessage::List->new(source => "cfg:$self->{name}",id => $self->{name}, instance_id => $self->{instance_id},%custom_attributes);
    return $list;
}

sub autocommit {
   my ($self,$auto) = @_;
   $self->{autocommit} = $auto;
}

sub save {
    my $self = shift;
    # this works because the list enforces a singleton behaviour on it's
    # data object, always returning the same one
    #
    if(exists($self->{native}) and $self->{native}) {
        $self->{native}->data()->commit();
    }
}

sub cclear {
   my $self = shift;
   my $native = $self->native();
   $native->data()->clear();
}

sub cinsert {
    my ($self,$user,$item,$value) = @_;
    my $native = $self->native();
    $native->data()->add_items($self->_as_item($item, $value));
}

sub cdelete {
    my ($self,$user,$item,$value) = @_;
    my $native = $self->native();
    $native->data()->remove_items($self->_as_item($item, $value));
}

sub cupdate {
    my ($self,$user,$item,$value) = @_;
    my $native = $self->native();
    $native->data()->remove_items($self->_as_item($item, $value));
    $native->data()->add_items($self->_as_item($item, $value));
}

sub _as_item {
    $_[1];
}

sub enumerate {
    my ($self,$sub,$change_id) = @_;
    # TODO we have to ignore the change_id for now 
   
    # nieve as heck implementation
    eval {
        my $data_object = $self->native()->data();
        &$sub($self->cfmt_clear()); # empty the destination
        my $new_sub = sub { 
                              my $item = shift;
                              &$sub($self->cfmt_insert('',$item,''));
                          };

        $data_object->enumerate_items($new_sub);
    };
    $self->error($@) if $@;
}

sub sync {
  my ($self,@args) = @_;
  my $result = $self->SUPER::sync(@args);
  if(exists($self->{attributes}->{source}) and $self->{attributes}->{source} =~ /^cdbfile/) {
    # then run makemap
    my $cmd = "$PREFIX/bin/pmx-makemap -l $self->{name}";
    system($cmd);
  }
  return $result;
}


# look at the local file and the pristine file to see if the 
# local file has changed since the last update
sub has_changed {
    my $self = shift;
    my @reg = stat $self->{filename};
    my @pris = stat $self->{pristine_filename};
    return 1 if $reg[9] > $pris[9]; # modification times
    return 1 if $reg[7] != $pris[7]; # different sizes
    return 0;
}

# copy the list/map to the pristine area
sub update_pristine {
    my $self = shift;
    my $filename = $self->{filename};
    my $prisname = $self->{pristine_filename};
    die "Can't find local copy of resource at $filename!" 
        unless -e $filename;
    copy($filename, $prisname) or 
        die "Can't copy $filename to $prisname: $!";
}

# diff the files and produce a list of changes
sub local_changes {
    my $self = shift;
    my $diff = diff $self->{pristine_filename}, 
                    $self->{filename}; 
    return $self->_diff_to_changes($diff, undef);
}

1;
