package PureMessage::Resources::Resource::multilist;
use strict;
use warnings;
use PureMessage::Resources::Resource::Base;
use base qw(PureMessage::Resources::Resource::Base);
use ActiveState::Handy qw/cp_tree/;
use File::Path qw/rmtree/;
use File::Find;
use Text::Diff;

return 1;

sub new {
    my($class, $store, @args) = @_;
    $class->SUPER::new($store, _lists => {}, @args);
}

sub can_incremental {
  return 1;
}


sub create_local {
    my $self = shift;
    require PureMessage::MultiList;
    return 1 if PureMessage::MultiList->Available($self->{name});
    # have to add it
    my $source_type =
        $self->{attributes}->{group_list} ? 'multigroupfile' : 'multifile';
    my $name = $self->{attributes}{name} || $self->{attributes}{description};
    my %list_config = ( id => $self->{name}, 
                        name => $name,
                        description => $self->{attributes}->{description},
                        precious => $self->{attributes}->{precious},
                        match_type => $self->{attributes}->{match_type},
                        source => "$source_type:$self->{name}",
                      );
    $list_config{comparator} = $self->{attributes}->{comparator} if $self->{attributes}->{comparator};
    PureMessage::MultiList->Setup(%list_config);
    return 1; # Setup will croak if there is a problem                  
}


sub _native_local {
    my $self = shift;
    require PureMessage::MultiList;
    my $list = PureMessage::MultiList->new($self->{name});
    $list;
}

sub _native_db {
    my $self = shift;
    require PureMessage::MultiList;
    my $list = PureMessage::MultiList->new(
        source=>"cfg:$self->{name}", instance_id => $self->{instance_id}
    );
    $list;
}

# multilists' own cacheing mechanism misbehaves...
sub _list {
    my ($self, $list) = @_;
    if($self->{_lists}->{$list}) {
        return $self->{_lists}->{$list};
    } else {
        my $native = $self->native();
        return $self->{_lists}->{$list} =
            $native->find_list(key => $list, edit_mode => 1);
    }   
}

sub cclear {
   my $self = shift;
   $self->native->enumerate_keys(sub { $self->_list(shift)->data->clear; });
}

sub cinsert {
    my ($self,$user,$item,$value) = @_;
    $self->_list($user)->data->add_items($item);
}

sub cdelete {
    my ($self,$user,$item,$value) = @_;
    $self->_list($user)->data->remove_items($item);
}

sub cupdate {
    my ($self,$user,$item,$value) = @_;
    $self->_list($user)->data->remove_items($item);
    $self->_list($user)->data->add_items($item);
}

sub save {
    my $self = shift;
    reset(%{$self->{_lists}});
    while(my($k, $v) = each(%{$self->{_lists}})) {
        $v->data->commit;
    }
}

sub enumerate {
    my ($self,$sub,$change_id) = @_;
    # TODO we have to ignore the change_id for now 
   
    # nieve as heck implementation
    eval {
        $sub->($self->cfmt_clear);

        $self->native->enumerate_keys(sub {
            my $user = shift;
            $self->_list($user)->data->enumerate_items(sub {
                $sub->($self->cfmt_insert($user, shift,''));
            });
        });
    };
    $self->error($@) if $@;
}

sub _aggregate_op {
  my ($self,$op,$user,$key,$value) = @_;
  if ($op eq 'I') {
      return ("$user->key = $value");
  }
}

# look at the local lists and the pristine lists to see if any 
# of the local lists has changed since the last update
sub has_changed {
    my $self = shift;
    my $reg = {};
    my $pris = {};

    my $wanted = sub {
        return unless -f $File::Find::name;
        my $filler = shift;
        (my $basename = $File::Find::name) =~ s#$File::Find::topdir/##;
        $filler->{$basename} = [ (stat($File::Find::name))[7,9] ];
    };

    find({wanted => sub { $wanted->($reg, @_)} , no_chdir => 1}, 
         $self->{filename});
    if (-d $self->{pristine_filename}) {
        find({wanted => sub { $wanted->($pris, @_)} , no_chdir => 1}, 
             $self->{pristine_filename});
    }

    for my $f (keys %$reg) {
        return 1 unless $pris->{$f};
        return 1 if $reg->{$f}[1] > $pris->{$f}[1]; # modification times
        return 1 if $reg->{$f}[0] != $pris->{$f}[0]; # different sizes
    }
    for my $f (keys %$pris) {
        return 1 unless $reg->{$f};
        return 1 if $reg->{$f}[1] > $pris->{$f}[1]; # modification times
        return 1 if $reg->{$f}[0] != $pris->{$f}[0]; # different sizes
    }
    return 0;
}

# copy the multilist to the pristine area
sub update_pristine {
    my $self = shift;
    my $list_dir = $self->{filename};
    my $pris_dir = $self->{pristine_filename};
    die "Can't find local copy of resource at $list_dir!" 
        unless -d $list_dir;
    rmtree $pris_dir;
    cp_tree $list_dir, $pris_dir;
}

# diff the files and produce a list of changes
sub local_changes {
    my $self = shift;

    my %sublists;
    my $wanted = sub {
        return unless -f $File::Find::name;
        (my $basename = $File::Find::name) =~ s#$File::Find::topdir/##;
        $sublists{$basename}++;
    };
    find({wanted => $wanted, no_chdir => 1}, 
         $self->{filename}, $self->{pristine_filename});

    # diff each of the sublists...
    my %changes = ('-' => [], '+' => []);
    for my $list (keys %sublists) {
        (my $listname = $list) =~ s#.+/##;
        my $diff;
        { # mask Text::Diff warnings for non-existant files
            local $SIG{__WARN__} = sub {};
            $diff = diff "$self->{pristine_filename}/$list", 
                            "$self->{filename}/$list"; 
        }
        my $ch = $self->_diff_to_changes($diff, $listname);
        push @{$changes{'-'}}, @{$ch->{'-'} || []};
        push @{$changes{'+'}}, @{$ch->{'+'} || []};
    }
    return \%changes;
}

1;




