package PureMessage::Resources::Resource::preferences;

use strict;
use warnings;
use PureMessage::Resources::Resource::Base;
use PureMessage::Resources::DBH qw(DBH);
use base qw(PureMessage::Resources::Resource::Base);

use PureMessage::Preferences;

return 1;

sub new {
    my($class, $store, @args) = @_;
    $class->SUPER::new($store, @args);
}

sub can_incremental {
  return 1;
}


sub create_local {
    my $self = shift;
    my $obj = $self->_native_local(writable => 1, create => 1);
    $obj->save();
    return 1;
}

sub _native_db {
    my $self = shift;
    $self->{dbh} = DBH();
    require PureMessage::Preferences;
    my $prefs = PureMessage::Preferences->new(
        storage_class => 'db', instance_id => $self->{instance_id},
        dbh => $self->{dbh}
    );
    $prefs;
}

sub _native_local {
    my $self = shift;
    my %opts = @_;
    $opts{writable} = $self->{store}->writable() unless exists($opts{writable});
    require PureMessage::Preferences;
    my $prefs = PureMessage::Preferences->new(
        instance_id => $self->{instance_id},
        %opts
    );
    $prefs;
}

# preferences don't support delete, and update / insert are the same thing.

sub save {
    my $self = shift;
    if($self->native->can('save')) {
        $self->native->save;
    }
}

sub cclear {
   my $self = shift;
   if(defined($self->native->{storage_class}) and $self->native->{storage_class} eq 'db') {
       $self->{dbh}->begin_work if($self->{dbh}->{AutoCommit});
       $self->{dbh}->do(
           "DELETE FROM cfg_kv WHERE instance_id = ?", {}, $self->{instance_id}
       ) or
           die "clearing instance id ", $self->{instance_id}, " failed: ",
               $self->{dbh}->errstr;
       $self->{dbh}->commit;
   }
}

sub cinsert {
    my ($self,$user,$item,$value) = @_;
    my $native = $self->native();
    die "unknown preferences key '$item'" unless($native->can($item));
    $native->$item($user, $value);
}

sub cupdate {
    my $self = shift;
    $self->cinsert(@_);
}

sub enumerate {
    my($self, $cb) = @_;
    
    eval {
        $cb->($self->cfmt_clear);
        $self->native->dump(sub {
            my($user, $prefs) = @_;
            reset(%$prefs);
            while(my($k, $v) = each(%$prefs)) {
                next if $k =~ /^_/; # ignore special housekeeping keys
                $cb->($self->cfmt_insert($user, $k, $v));
            }
        });
    };
    
    $self->error($@) if($@);
}


sub _aggregate_op {
  my ($self,$op,$user,$key,$value) = @_;
  if ($op eq 'I') {
      return ("$user->$key = $value");
  }
}

1;


