package PureMessage::Resources::Resource::session;

use strict;
use warnings;
use PureMessage::Resources::Resource::Base;
use PureMessage::Resources::DBH qw(DBH);
use PureMessage::Version qw($PREFIX);
use base qw(PureMessage::Resources::Resource::Base);
use CGI::Session;
use File::Path qw(mkpath);

return 1;

sub new {
    my($class, $store, @args) = @_;
    $class->SUPER::new($store, @args);
}

sub _native_db {
    my $self = shift;
    my $dbh;
    unless(exists($self->{attributes}) and scalar(keys(%{$self->{attributes}}))) {
      require PureMessage::Resources::Manager;
      my $temp = PureMessage::Resources::Manager->Find($self->{name});
      $self->{attributes} = $temp->{attributes};
    } 
    if ($self->{dbh}) {
       $dbh = $self->{dbh};
    } else {
       require PureMessage::Resources::DBH;
       $dbh = PureMessage::Resources::DBH->DBH();
    };

    my $create = sub {
                        my $session_id = shift;
                        my $args = shift;
                        my $driver = $self->{attributes}->{driver_db}; 
                        $args->{Handle} = $dbh;
                        $args->{TableName} = 'session';
                        my $obj = CGI::Session->new($driver,$session_id,$args);
                      
                        # because it is a #duration type it will be seconds, make it in the format
                        # CGI::Session expects
                        $obj->expire($args->{session_expire}) if defined $args->{session_expire};
                        $obj->expire( $_ => $args->{expire}->{$_}) foreach keys %{$args->{expire}};
                        return $obj;

                     };

    my $SQL_ENUMERATE_SESSIONS = "select id from session";

    my $list = sub {
                      my @sessions = ();
                      my $sth = $dbh->prepare($SQL_ENUMERATE_SESSIONS);
                      $sth->execute();
                      while(my $h = $sth->fetchrow_hashref()) {
                        push(@sessions,$h->{id});
                      }
                      return \@sessions;
                    };

    my $obj = PureMessage::Resources::Session::_wrapper->new($create,$list);
    return $obj;
}

sub _native_local {
    my $self = shift;

    unless($self->{attributes}) {
      require PureMessage::Resources::Manager;
      my $temp = PureMessage::Resources::Manager->Find($self->{name});
      $self->{attributes} = $temp->{attributes};
    } 
    
    my $directory = $self->{attributes}->{Directory};
    my ($mask,$old_umask);
    $mask = defined($self->{attributes}->{'umask'}) ? oct($self->{attributes}->{'umask'}) : 077;
    $old_umask = umask $mask;
    $directory = "$PREFIX/$directory" unless $directory =~ m(^/);
    mkpath($directory) unless -e $directory;
    umask $old_umask;
                        
    die "error_session_directory:$directory" unless -d $directory and -w $directory;


    my $create = sub {
                        my $session_id = shift;
                        my $args = shift;
                        my $driver = $self->{attributes}->{driver_local}; 
                        $args->{Directory} = $directory; 
                        $old_umask = umask $mask;
                        my $obj = CGI::Session->new($driver,$session_id,$args);
                      
                        # because it is a #duration type it will be seconds, make it in the format
                        # CGI::Session expects
                        $obj->expire($args->{session_expire}) if defined $args->{session_expire};
                        $obj->expire( $_ => $args->{expire}->{$_}) foreach keys %{$args->{expire}};
                        umask $old_umask;
                        return $obj;

                     }; 

    my $list    = sub {
                             my @sessions = ();
                             opendir DIR, $directory || die "Can't open Directory '$directory': $!";
                             while(my $sf = readdir DIR) {
                                next unless -f "$directory/$sf";
                                next unless $sf =~ /^cgisess_(.+)/;
                                push(@sessions,$1);
                             }
                             closedir DIR;
                             return \@sessions;
                        };

    my $obj = PureMessage::Resources::Session::_wrapper->new($create,$list);
    return $obj;
}


sub save {
    my $self = shift;
}

sub cclear {
   my $self = shift;
   my $native = $self->native();
   my $sessions = $native->sessions();
   foreach (my $sid = @$sessions) {
      my $session = $native->create_session($sid);
      $session->delete();
   }
}

sub cinsert {
    my ($self,$user,$item,$value) = @_;
    my $native = $self->native();
    my $session = $native->create_session($user);
    $session->param($item,$value);
    $session->flush();
}

sub cupdate {
    my $self = shift;
    $self->cinsert(@_);
}

sub cclone {
    my $self = shift;
    my ($sid,$from) = @_;
    eval {
       my $dest = $self->native();
       my $sess = $dest->create_session(undef);
       my $ref =  $from->{_DATA};
       $sess->store($sid,$sess->{_OPTIONS},$from->{_DATA});
       $sess->delete();
    };
}


sub enumerate {
    my ($self,$sub,$change_id) = @_;
    
    eval {
        &$sub($self->cfmt_clear);
        my $native = $self->native();
        my $sids = $native->sessions();
        foreach my $sid (@$sids) {
          my $sess = $native->create_session($sid);
          &$sub($self->cfmt_clone($sid,$sess));
        }
    };
    
    $self->error($@) if($@);
}

sub _aggregate_op {
  my ($self,$op,$sid,$obj,$ignore) = @_;
  
  if ($op eq 'L') { # we only see session data as Clone me data
    my $h = $obj->param_hashref();
    my @data = ($sid);
    my $o = (' ' x 5) . '-> ';
    push(@data,"$o email = $h->{email}") if exists($h->{email});
    push(@data,"$o authorized = $h->{_authorized}") if exists($h->{_authorized});
    push(@data,"$o created (sec. ago)= " . (time - $obj->ctime()) );
    push(@data,"$o expires in (sec.)= " . $obj->expire());
    return @data;
  }
}


1;

package PureMessage::Resources::Session::_wrapper;

sub new {
   my ($CLASS,$factory,$list,%options) = @_;
   my $obj = bless(\%options,$CLASS);
   $obj->{_factory} = $factory;
   $obj->{_list} = $list;
   return $obj;
}

sub create_session {
   my ($self,$session_id,$args) = @_;
   $self->{_factory}($session_id,$args);
}

sub sessions {
   my $self = shift;
   return $self->{_list}();
}

1;




