package PureMessage::Resources::Store::Base;

=head1 NAME

PureMessage::Resources::Store::Base - Base Class for Resource Stores

=head1 DESCRIPTION

The C<PureMessage::Resources::Manager> is a base class only and should not
be used directly.  The following methods are required to be overloaded
by inherited classes:

=over

=item new(%options)

Create a new class.

=cut


sub new {
   my ($CLASS,%options);
   bless($CLASS,\%options);
}

sub begin_batch {
}

sub end_batch {
}

=item enumerate($resource_manager,sub{})

Iterate throught all available resource instances in this store. The passed
sub will be called for each found resource with a resource object.


=cut

sub enumerate {
 return;  
}

sub writable {
  my $self = shift;
  $self->{writable} = $_[0] if scalar(@_) == 1;
  return $self->{writable};
}

=item find($resource_manager,$resource_name)

Returns the named resource object if found, or undef.

=cut

# this will enforce singleton behaviour for resources. We need this
# for some resources that need be refreshed when manipulated
sub find { # $resource_name
  my ($self,$resource_name) = @_;
  if (exists($self->{FOUND_RESOURCES}->{$resource_name}) and $self->{FOUND_RESOURCES}->{$resource_name}) {
      return $self->{FOUND_RESOURCES}->{$resource_name};
  }
  return;
}

# register a resource as found, this is an internal method
sub found {
  my ($self,$resource) = @_;
  # disable caching
  #$self->{FOUND_RESOURCES}->{$resource->name()} = $resource;
  $resource;
}

# remove a resource from the list of resources we have cached copies of
sub forget {
  my ($self,$resource) = @_;
  my $name = ref($resource) ? $resource->name() : $resource;
  delete($self->{FOUND_RESOURCES}->{$name});
}


=item add($resource_manager, $name)

Create a local instance of the named resource. If the named resource
already exists it is returned instead.

=cut

sub add { # $resource_manager, $name
}


1;


=back

=head1 COPYRIGHT

Copyright (C) 2004, Sophos PLC

=head1 SEE ALSO
 
 none
  
=cut



